import yaml
import ujson
from typing import TextIO, Iterable

from .message import Message


def format_json(fp: TextIO, messages: Iterable[Message], indent: int = 2) -> None:
    ujson.dump([m.__dict__ for m in messages], fp, ensure_ascii=False, indent=indent)


def format_yaml(fp: TextIO, messages: Iterable[Message], indent: int = 2) -> None:
    yaml.safe_dump([m.__dict__ for m in messages], fp, allow_unicode=True, indent=indent)


formatters = {
    'json': format_json,
    'yaml': format_yaml,
}



import click
from pathlib import Path

from . import parse
from .formatter import formatters


@click.command()
@click.option('--indent', '-d', prompt='Output indent', help='Output file indent.', type=int, default=2, prompt_required=False)
@click.option('--input-file', '-i', prompt='Input file path', help='Input file path.',
              type=click.Path(exists=True, dir_okay=False, path_type=Path))
@click.option('--output-file', '-o', prompt='Output file path(format will be filled automatically)',
              help='Output file path.', type=click.Path(dir_okay=False, path_type=Path), default='output',
              prompt_required=False)
@click.option('--output-format', '-f', prompt='Output format', help='Output file format.',
              type=click.Choice(['json', 'yaml']), default='json')
def run(input_file: Path, output_file: Path, output_format: str, indent: int) -> None:
    if not output_file.suffix:
        output_file = output_file.with_suffix(f'.{output_format}')
    lines = input_file.read_text('utf8').splitlines()
    with output_file.open('w', encoding='utf8') as fp:
        formatters[output_format](fp, parse(lines), indent)



from .parser import parse as parse
from .message import Message as Message
from .formatter import (
    formatters as formatters,
    format_yaml as format_yaml,
    format_json as format_json,
)


__all__ = [
    'parse',
    'Message',
    'formatters',
    'format_yaml',
    'format_json',
]
